(* ::Package:: *)

(*:Mathematica Version: 6.0 *)

(*:Name: Kinetics *)

(*:Title: Mass Action Law *)

(*:Authors:
		Rui Dil\[ATilde]o and Daniele Muraro
		IST, Nonlinear Analysis Group
		Department of Physics
		Av. Rovisco Pais, 1
		1049-001 Lisboa Codex
		Portugal

		phone:    351-218417617/8
		fax:      351-218419123
		e-mail:   rui@sd.ist.utl.pt, muraro@sd.ist.utl.pt *)

(*:Requirements: none. *)

(*:Observations: This is the working package with all the code commented. 
For the release version, it is necessary to convert this file to the 
Format Style Code. *)


BeginPackage["GeneticNetworks`"]


(* Genetic Networks *)
GeneticNetworks::usage="GeneticNetworks.m is a package that models a gene regulatory network. 
The following functions are defined in the GeneticNetworks package:

NetworkGraph 
Reactions
SubstanceNames
SubstanceVariables
SubstanceInitialConditions
ParameterNames
ParameterInput
ReactionGraph
Equations
ConservationLaws

The first functions to run are NetworkGraph[activation, repression] and Reactions[activation, repression].
The input are two tables that indicate the activations and repressions of proteins on genes.
For example,
activation={A \[Rule] B, B \[Rule] C}
repression={B \[Rule] A}.
This means that the protein A activates the gene B, the protein B activates the gene C, and the protein B represses the gene A (please note that 
the same symbol is used to denote both a gene and the corresponding protein).

NetworkGraph[activation, repression] returns the graph of the activations (green arrow) and repressions (red edge).

Reactions[activation, repression] prints the names of the proteins of the graph; the number of activation, repression and total sites of the 
corresponding gene; the name of the proteins which have no sites and that, by default, will be considered constant.
This function also returns two tables, reactions and parameters, containing all the reactions and the parameters of the graph accounting of the 
potential production and degradation of the proteins.
Genes are denoted by the name of the protein they produce and with subscripts and overscripts indicating their regulator binding sites.
The subscripts are reserved to proteins which repress the gene, while the overscripts to proteins which activate it.
   
SubstanceNames[reactions], SubstanceVariable[reactions], SubstanceInitialConditions[reactions] return, respectively, a table with the name of 
the variables in the table reactions, the same table with the explicit temporal dependence (if any), the table with the substance variables 
taken at time t=0 and a set of global variables in order to assign externally initial conditions.

ParameterNames[reactions], ParameterInput[parameters] compute, respectively, a table with the name of the parameters in the reactions 
independently of the graph of activations and repressions, and the same table with a set of global variables in order to assign parameter values.

ReactionGraph[reactions, parameters] displays the reactions with the corresponding parameters.

Equations[reactions, parameters] calculates the differential equations describing the time evolution of the concentrations of genes and proteins
of the genetic network. 
The differential equations are obtained by the mass action law, and the command Equations[reactions, parameters] returns a table with the 
differential equations.

ConservationLaws[reactions] returns a table with the conservation laws of the full set of reactions."

NetworkGraph::usage="NetworkGraph[activation, repression] returns the graph of a genetic regulatory network.
The input are two tables that indicate the activations and repressions of proteins on genes.
For example,
activation={A \[Rule] B, B \[Rule] C}
repression={B \[Rule] A}.
This means that the protein A activates the gene B, the protein B activates the gene C, and the protein B represses the gene A (please note that 
the same symbol is used to denote both a gene and the corresponding protein).
The activation are drawn with a green arrow, while the repressions are drawn with a red edge."
ManipulateGraph::usage="ManipulateGraph[activation, repression] returns the graph of a genetic regulatory network in a form that can be 
manipulated. The input is the same of the function NetworkGraph"
ReactionsNewModel::usage="ReactionsNewModel[activation_, repression_] :=..."
Reactions::usage="Reactions[activation, repression] returns the reactions and the parameters involved in a genetic network. It also prints the 
names of the proteins and the numbers of the regulatory sites of the genes in the network. 
The input are two tables that indicate the activations and repressions of proteins on genes.
For example,
activation={A \[Rule] B, B \[Rule] C}
repression={B \[Rule] A}.
This means that the protein A activates the gene B, the protein B activates the gene C, and the protein B represses the gene A (please note that 
the same symbol is used to denote both a gene and the corresponding protein).
Reactions prints the names of the proteins of the graph; the number of activation, of repression and of the total sites of the corresponding gene; 
the name of the proteins which have no sites and that, by default, will be considered constant.
This function then returns two tables, reactions and parameters, containing all the reactions and the parameters of the graph accounting of the 
potential production and degradation of the proteins.
Genes are denoted by the name of the protein they produce and with subscripts and overscripts indicating their regulator binding sites.
The subscripts are reserved to proteins which repress the gene, the overscripts to proteins which activate it.
The network of interactions can be divided into three different species, each stored in a global table: 
ReactionGeneProtein, ReactionProduction, ReactionDegradation.
The first table holds the double bimolecular reactions between a gene and a protein activating or repressing it, i.e. 
\!\(\*SubscriptBox[SuperscriptBox[\"B\", \"0\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) + A \[LeftRightArrow] \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"A\"], 
RowBox[{\"0\", \",\", \"0\"}]]\)
The second table holds the reactions of production of a protein by an activated gene, i.e. 
\!\(\*SubscriptBox[SuperscriptBox[\"B\", \"A\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) \[Rule] \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"A\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) + B.
The third table holds the degradations of proteins (if any), i.e. B \[Rule] .
Correspondingly, the following global tables for parameters are defined: ParameterFw, ParameterBw, ParameterProduction, ParameterDegradation.
ParameterFw and ParameterBw hold the parameters related to ReactionGeneProtein. The parameters are denoted by the small letter of the gene 
involved in the reaction and, in the index, the small letter of the protein reacting with the gene.
For the backward reaction the index is preceded by a minus sign. For example,
for the forward reaction \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"0\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) +A \[Rule] \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"A\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) the corresponding parameter is \!\(\*SubscriptBox[\"b\", \"a\"]\),
for the backward reaction \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"A\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) \[Rule] \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"0\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) +A the corresponding parameter is \!\(\*SubscriptBox[\"b\", 
RowBox[{\"-\", \"a\"}]]\).
ParameterProduction holds the parameters related to ReactionProduction denoted by the letter p and, in the index, the name of the protein 
involved in the reaction. For example, 
for the reaction \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"A\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) \[Rule] \!\(\*SubscriptBox[SuperscriptBox[\"B\", \"A\"], 
RowBox[{\"0\", \",\", \"0\"}]]\) + B the corresponding parameter is \!\(\*SubscriptBox[\"p\", \"B\"]\).
ParameterDegradation holds the parameters related to ReactionDegradation denoted by the letter d and, in the index, the name of the protein 
involved in the reaction. For example, 
for the reaction B \[Rule] the corresponding parameter is \!\(\*SubscriptBox[\"d\", \"B\"]\)."
ReactionsOneSite::usage="ReactionOneSite[activation, repression] returns the same prints and tables of Reactions, but counting only one binding 
site per gene. The proteins have to compete to bind the gene."
SubstanceNames::usage="SubstanceNames[reactions] returns a table with the name of the variables in the table reactions."
SubstanceVariables::usage="SubstanceVariables[reactions] returns a table with the name of the variables in the table reactions, with the explicit 
temporal dependence --- Name[t]."
SubstanceInitialConditions::usage="SubstanceInitialConditions[reactions] returns a table with the substance variables taken at time t=0, and a 
set of global variables in order to assign externally initial conditions. The global variables are InitCond[i]."
ParameterNames::usage="ParameterNames[reactions] returns a table with the name of the parameters in the reactions. In order to facilitate the 
symbolic manipulation and the attribution of values, the first group of parameters in the output table corresponds to the left-to-right reactions. The second group of parameters corresponds to the rate constants of the right-to-left reactions." 
ParameterInput::usage="ParameterInput[parameters] returns a table with the name of the parameters in the reactions, and a set of global variables 
in order to assign parameter values. The global variables are ParInputVar[i]."
ReactionGraph::usage="ReactionGraph[reactions] displays the reactions with the corresponding rate constants."
Equations::usage="Equations[reactions, parameters] calculates the differential equations describing the time evolution of the concentrations of 
the chemical substances involved in the set of chemical reactions of a genetic network. 
The differential equations are obtained by the mass action law, and the command Equations[reactions, parameters] returns a table with the 
differential equations.
The input to Equations[reactions, parameters] are the tables of the reactions and of the corresponding parameters obtained by the function 
Reactions[activation, repression]. 
The proteins with a constant concentration in the output of Reactions will be considered, by default, constant. Thus, for such proteins, there 
will be no differential equation. 
Another feature of this package is the introduction of 'garbage' variables, denoted by \!\(\*SubscriptBox[\"G\", \"i\"]\)(i=1,...,n), to compute 
also the degradations with the mass action law. For example,
the reaction B \[Rule]  will be substituted by B \[Rule] \!\(\*SubscriptBox[\"G\", \"1\"]\). The equations with these garbage variables must not be considered."
ConservationLaws::usage="ConservationLaws[reactions] returns a table with the conservation laws of the full set of reactions. 
The proteins with a constant concentration in the output of Reactions will be considered, by default, constant. 
Thus, such proteins should appear among the conservation laws. 
Another feature of this package is the introduction of 'garbage' variables, denoted by \!\(\*SubscriptBox[\"G\", \"i\"]\)(i=1,...,n), to compute 
also the degradations with the mass action law. For example,
the reaction B \[Rule]  will be substituted by B \[Rule] \!\(\*SubscriptBox[\"G\", \"1\"]\). The conservation laws with these garbage variables must not be 
considered."
CreateNB::usage="CreateNB[] creates the notebook Equations, and prints the tables equations, substanceNames, substanceInitialCondition, 
substanceVariables. It is automatically run in Equations and it is necessary to make these tables recognized by the Mathematica function 
Symbolize.  "
FillParameters::usage="FillParameters[start,finish] compares all the elements of the table parameters with the ones in the positions between 
start and finish. Then gives the same value to the parameters with the same name."
EmptyGenes::usage="EmptyGenes[value] assigns the value in input to the initial conditions of every empty gene."
NotEmptyGenes::usage="NotEmptyGenes[value] assigns the value in input to the initial conditions of every not empty gene."


(* Euler *)
EulerIntegration::usage="EulerIntegration[equations_, parameters_, initialConditions_, variables_, iterations_, step_] returns a table with the temporal evolution of the equations in input."
EulerPoints::usage="EulerPoints[equations_, parameters_, initialConditions_, variables_, iterations_, step_] returns the final points of the variables after the evolution of the equations 
in input."
CEquations::usage="CEquations[equations, substanceVariables, parameters] generates the output file 'equations.h' where the equations of the system are translated into a C function."
WriteDataC::usage="WriteDataC[] generates the output file 'data.txt' with the parameters and the initial conditions used in the current network."
ParametersWithoutRepetition::usage="..."
EmptyGenesPosition::usage="EmptyGenesPosition[substanceInitialConditions, emptyGenes] returns the positions of the empty genes in the table 'substanceInitialConditions'"


(* Kinetics *)
EquationsKinetics::usage="EquationsKinetics[reactions] calculates the differential equations describing the time evolution of the concentrations 
of several chemical substances involved in a set of chemical reactions. The differential equations are obtained by the mass action law, and the 
command EquationsKinetics[reactions] returns a table with the differential equations. 
The input to EquationsKinetics[reactions] is a table. For example, 
reactions={A+B \[Rule] 2 D1,   D1+H1 \[Rule] R1+3*R3, A+S \[LeftRightArrow] D1}.
The program generates rate constants as global variables. The rate 
constants are denoted by \!\(\*SubscriptBox[\"k\", 
RowBox[{\"\[PlusMinus]\", \"i\"}]]\), where i is the ith reaction, in the input table.
Another feature of this package is to consider the cases where the concentration
of some chemical components are constant over time (open reactor reactions). For example, in the reactions,
reactions={A$+B \[Rule] 2 D1,   D1+H1 \[Rule] R1+3*R3}
the symbols $ indicates that the substance A must be treated as a constant during the reaction, and therefore there will be no differential 
equation for A. 
If a substance appears several times in the table reactions but it is marked only once with the symbol $, all the context where the substance 
appears will be treated as a constant concentration substance.
The package Kinetics has eight commands:
EquationsKinetics[reactions], ConservationLawsKinetics[reactions], ReactionGraphKinetics[reactions], SubstanceNamesKinetics[reactions], 
SubstanceVariablesKinetics[reactions], SubstanceInitialConditionsKinetics[reactions], ParameterNamesKinetics[reactions], 
ParameterInputKinetics[reactions].
Global variables for manipulation of parameters and initial conditions:
InitCond[i], ParInputVar[i]."
ReactionGraphKinetics::usage="ReactionGraphKinetics[reactions] displays the reactions with the corresponding rate constants."
ConservationLawsKinetics::usage="ConservationLawsKinetics[reactions] returns a table with the conservation laws of the full set of 
reactions. 
In the open reactor case, the concentration of the marked substances are constant over time, and the characters representing these substances 
appear in the lists of conservation laws and parameters."
SubstanceNamesKinetics::usage="SubstanceNamesKinetics[reactions] returns a table with the name of the variables in the table reactions."
SubstanceVariablesKinetics::usage="SubstanceVariablesKinetics[reactions] returns a table with the name of the variables in the table reactions, 
with the explicit temporal dependence --- Name[t]."
ParameterNamesKinetics::usage="ParameterNamesKinetics[reactions] returns a table with the name of the parameters in the reactions. 
In order to facilitate the symbolic manipulation and the attribution of values, the first group of parameters in the output table corresponds 
to the left-to-right reactions. The second group of parameters corresponds to the rate constants of the right-to-left reactions. The third group 
of parameters are the concentrations of the substances for the open reactor cases."
ParameterInputKinetics::usage="ParameterInputKinetics[reactions] returns a table with the name of the parameters in the reactions, and a set of 
global variables in order to assign parameter values. The global variables are ParInputVar[i]."
SubstanceInitialConditionsKinetics::usage="SubstanceInitialConditionsKinetics[reactions] returns a table with the substance variables taken at 
time t=0, and a set of global variables in order to assign externally initial conditions. The global variables are InitCond[i]."


Begin["`Private`"]

<<Combinatorica`
<<GraphUtilities`
Notation`AutoLoadNotationPalette = False;
<<Notation`


(* GENETIC NETWORKS *)


NetworkGraph[activation_, repression_] :=
Block[{act,rep,var,varString,ind1,ind2,actpar,reppar,actw,repw,defgraph,g,h},
    
act=activation;
rep=repression;

(* Converts activations and repressions into strings *)
var=Union@Flatten@Table[{Variables@act[[i,1]],Variables@act[[i,2]]},{i,Length@act}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
act=act/.Table[var[[i]]->varString[[i]],{i,Length@var}];
var=Union@Flatten@Table[{Variables@rep[[i,1]],Variables@rep[[i,2]]},{i,Length@rep}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
rep=rep/.Table[var[[i]]->varString[[i]],{i,Length@var}];

(* Computes the parameters *)
ind1=Table[ToLowerCase[act[[i,1]]],{i,Length@act}];
ind2=Table[ToLowerCase[act[[i,2]]],{i,Length@act}];
actpar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@act}];
ind1=Table[ToLowerCase[rep[[i,1]]],{i,Length@rep}];
ind2=Table[ToLowerCase[rep[[i,2]]],{i,Length@rep}];
reppar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@rep}];
actw=Table[{act[[i]],actpar[[i]]},{i,Length@act}];
repw=Table[{rep[[i]],reppar[[i]]},{i,Length@rep}];
Global`graph=Join[actw,repw];

(* Global`graph ??? *)

(* Defines and plots the graph *)
h=Graphics[Line[{{-1,-1/2},{-1,1/2}}]];
defgraph=({If[MemberQ[EdgeList[act],#2],{Green,Thickness[0.001],Arrowheads[.05],Arrow[#]},{Red,Arrowheads[{{.05,1,{h,1}}}],Arrow[#]}],If[#3=!=None,Text[#3,Mean[#1]],{}]}&);
g=GraphPlot[Global`graph,DirectedEdges->True,EdgeRenderingFunction->defgraph,VertexLabeling->True];
Return[g];
];


ManipulateGraph[activation_, repression_,imagesize_] :=
Block[{},
   
act=activation;
rep=repression;
imSize=imagesize;

(* Converts activations and repressions into strings *)
varAct=Union@Flatten@Table[{Variables@act[[i,1]],Variables@act[[i,2]]},{i,Length@act}];
varString=varAct/.Table[varAct[[i]]->ToString@varAct[[i]],{i,Length@varAct}];
var=varString;
act=act/.Table[varAct[[i]]->varString[[i]],{i,Length@varAct}];
varRep=Union@Flatten@Table[{Variables@rep[[i,1]],Variables@rep[[i,2]]},{i,Length@rep}];
varString=varRep/.Table[varRep[[i]]->ToString@varRep[[i]],{i,Length@varRep}];
var=Union@Flatten@AppendTo[var,varString];
rep=rep/.Table[varRep[[i]]->varString[[i]],{i,Length@varRep}];

(* Computes the parameters *)
ind1=Table[ToLowerCase[act[[i,1]]],{i,Length@act}];
ind2=Table[ToLowerCase[act[[i,2]]],{i,Length@act}];
actpar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@act}];
ind1=Table[ToLowerCase[rep[[i,1]]],{i,Length@rep}];
ind2=Table[ToLowerCase[rep[[i,2]]],{i,Length@rep}];
reppar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@rep}];
actw=Table[{act[[i]],actpar[[i]]},{i,Length@act}];
repw=Table[{rep[[i]],reppar[[i]]},{i,Length@rep}];
Global`graph=Join[actw,repw];

(* Prepares some parameters and functions of the graph *)
v=Length@var;
h=Graphics[Line[{{-1,-1/2},{-1,1/2}}]];
AnnotatedEdge[p1_,q1_,label1_]:={Arrowheads[{{.015,.5,Graphics[Inset[Style[label1,Medium],{Center,Top}]]},{.05,1.,{h,1}}}],Arrow[{p1,q1},0.1]};
AnnotatedArrow[p2_,q2_,label2_]:={Arrowheads[{{.015,.5,Graphics[Inset[Style[label2,Medium],{Center,Top}]]},{.05,1.}}],Arrow[{p2,q2},0.2]};
a=Flatten@{act[[All,1]],act[[All,2]]};
LA=Length@act;
r=Flatten@{rep[[All,1]],rep[[All,2]]};
LR=Length@rep;
AppearanceVar=Table[Style[ToString[var[[i]]],Large],{i,v}];
grid=Flatten[Table[{{1,2 i-1},{v,2 i-1}},{i,IntegerPart[v/2]}],1];
GridHeigth=2 IntegerPart[v/2];
If[FractionalPart[v/2]!=0,AppendTo[grid,{1,++GridHeigth}];GridHeigth++];

(* Plots the graph *)
g = DynamicModule[
{p=grid,rad=0.3},
LocatorPane[Dynamic[p],Dynamic@Graphics[{{Thick,Green,Table[If[a[[i]]==a[[i+LA]],{Circle[{p[[Flatten@Position[var,a[[i]]]]][[1,1]]+
rad,p[[Flatten@Position[var,a[[i]]]]][[1,2]]},rad] ,Inset[Style[actpar[[i]],FontSize->20],{p[[Flatten@Position[var,a[[i]]]]][[1,1]]+
rad,p[[Flatten@Position[var,a[[i]]]]][[1,2]]}],Arrowheads[0.04],Arrow[{p[[Flatten@Position[var,a[[i]]]]][[1]],p[[Flatten@Position[var,a[[i]]]]][[1]]+
{-0.00025,0.001}}]},{AnnotatedArrow[p[[Flatten@Position[var,a[[i]]]]][[1]],p[[Flatten@Position[var,a[[i+LA]]]]][[1]],Style[actpar[[i]],FontSize->20]]}],{i,LA}]},
{Thick,Red,Table[If[r[[i]]==r[[i+LR]],{Circle[{p[[Flatten@Position[var,r[[i]]]]][[1,1]]+rad,p[[Flatten@Position[var,r[[i]]]]][[1,2]]},rad] ,
Inset[Style[reppar[[i]],FontSize->20],{p[[Flatten@Position[var,r[[i]]]]][[1,1]]+rad,p[[Flatten@Position[var,r[[i]]]]][[1,2]]}],
Arrowheads[0.04],Arrow[{p[[Flatten@Position[var,r[[i]]]]][[1]],p[[Flatten@Position[var,r[[i]]]]][[1]]+{-0.00025,0.001}}]},
{AnnotatedEdge[p[[Flatten@Position[var,r[[i]]]]][[1]],p[[Flatten@Position[var,r[[i+LR]]]]][[1]],Style[reppar[[i]],FontSize->20]]}],{i,LR}]}},
PlotRange->{{0,Length@var+1},{0,GridHeigth}},ImageSize->imSize, Frame->True,FrameTicks->True],Appearance->AppearanceVar]];

Return[g];

];


Reactions[activation_, repression_] :=
Block[{act,rep,var,varString,ActSites,RepSites,ProteinsWithoutGenesSites,RTot,ParTot,Sites,reacs,Elem,PosAct,PosRep,act1,rep1,X,Y,perm,tmp,inter,PAct,PRep,Act0,Rep0,r,r1,r2,J,el1,el2,ParFw,ParBw,Par,
interact,interrep,sub, ElNotSymbolized},
 
act=activation;
rep=repression;

p=Global`p;
d=Global`d;
El=Global`El;
graph=Global`graph;

Global`FirstTime=1;

(* Converts activations and repressions into strings *)
var=Union@Flatten@Table[{Variables@act[[i,1]],Variables@act[[i,2]]},{i,Length@act}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
act=act/.Table[var[[i]]->varString[[i]],{i,Length@var}];
var=Union@Flatten@Table[{Variables@rep[[i,1]],Variables@rep[[i,2]]},{i,Length@rep}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
rep=rep/.Table[var[[i]]->varString[[i]],{i,Length@var}];

(* Computes the parameters *)
ind1=Table[ToLowerCase[act[[i,1]]],{i,Length@act}];
ind2=Table[ToLowerCase[act[[i,2]]],{i,Length@act}];
actpar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@act}];
ind1=Table[ToLowerCase[rep[[i,1]]],{i,Length@rep}];
ind2=Table[ToLowerCase[rep[[i,2]]],{i,Length@rep}];
reppar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@rep}];
actw=Table[{act[[i]],actpar[[i]]},{i,Length@act}];
repw=Table[{rep[[i]],reppar[[i]]},{i,Length@rep}];
graph=Join[actw,repw];

ElNotSymbolized=El=VertexList[graph];	
Do[Symbolize[ ParsedBoxWrapper [ SubsuperscriptBox[El[[i]],"_","_"] ] ],{i,Length@El}];
Do[Symbolize[ ParsedBoxWrapper [ SubscriptBox[El[[i]],"_"] ] ],{i,Length@El}];
(* Do[Symbolize[ ParsedBoxWrapper [ SuperscriptBox[El[[i]],"_"] ] ],{i,Length@El}]; *)
 
(* Converts activations and repressions into strings *)
var=Union@Flatten@Table[{Variables@act[[i,1]],Variables@act[[i,2]]},{i,Length@act}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
act=act/.Table[var[[i]]->varString[[i]],{i,Length@var}];
var=Union@Flatten@Table[{Variables@rep[[i,1]],Variables@rep[[i,2]]},{i,Length@rep}];    
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
rep=rep/.Table[var[[i]]->varString[[i]],{i,Length@var}];

	
ActSites=Table[0,{Length[El]}];
Do[
   Do[
      If[El[[j]]==act[[i,2]],ActSites[[j]]++]
   ,{i,Length[act]}];
,{j,Length[El]}];

RepSites=Table[0,{Length[El]}];
Do[
   Do[
      If[El[[j]]==rep[[i,2]],RepSites[[j]]++]
   ,{i,Length[rep]}];
,{j,Length[El]}];

Sites=ActSites+RepSites;

           Print["Proteins = ",El];
   Print[];Print["Activation Sites per Gene = ",ActSites];
   Print[];Print["Repression Sites per Gene = ",RepSites];
   Print[];Print["Total Sites per Gene = ",Sites];

ProteinsWithoutGenesSites={};ProteinsWithoutGenes={};
Do[
   If[Sites[[i]]==0,
      ProteinsWithoutGenesSites=Append[ProteinsWithoutGenesSites,1];
      ProteinsWithoutGenes=Append[ProteinsWithoutGenes,El[[i]]],
      ProteinsWithoutGenesSites=Append[ProteinsWithoutGenesSites,0]
     ](* End If *)
,{i,Length@Sites}];(* End Do *)
Print[];Print["Proteins Without Genes = ", ProteinsWithoutGenes];

ConstantProteins={};
Do[
   If[StringMatchQ[ProteinsWithoutGenes[[i]],__~~"$"],
      ConstantProteins=Append[ConstantProteins,ProteinsWithoutGenes[[i]]]
     ]
,{i,Length@ProteinsWithoutGenes}];
Print[];Print["Constant Proteins = ", ConstantProteins];



X=Table[Table[0,{i,ActSites[[j]]}],{j,Length@El}];
Y=Table[Table[0,{i,RepSites[[j]]}],{j,Length@El}];

Global`emptyGenes={};
Do[
   If[!MemberQ[ProteinsWithoutGenes,El[[j]]],
      AppendTo[Global`emptyGenes, Subscript[ToExpression@El[[j]], Row[Y[[j]],","]]^Row[X[[j]],","]]
     ];
,{j,Length@El}];

Global`emptyGenesNotSymbolized={};
Do[
   If[!MemberQ[ProteinsWithoutGenes,ElNotSymbolized[[j]]],
      AppendTo[Global`emptyGenesNotSymbolized, Subscript[ToExpression@ElNotSymbolized[[j]], Row[Y[[j]],","]]^Row[X[[j]],","]]
     ];
,{j,Length@ElNotSymbolized}];


(* Loop per gene *)
ReactionsGeneProtein={};
ParametersFw={};   
ParametersBw={};   
ReactionsProduction={};
ParametersProduction={};
ReactionsDegradation={};
ParametersDegradation={};
reacs={};
RTot={};
ParTot={};
Do[
   Elem=El[[gene]];

   PosAct=Position[act,Elem];
   PosRep=Position[rep,Elem];
  
   (* Activation proteins *)
   act1={};
   Do[
      If[PosAct[[i,2]]==2,
         act1=Append[act1,act[[PosAct[[i,1]],1]]]
        ](* End If *)
   ,{i,Length@PosAct}];(* End Do *)
 
   (* Repression proteins *)
   rep1={};
   Do[
      If[PosRep[[i,2]]==2,
         rep1=Append[rep1,rep[[PosRep[[i,1]],1]]];
        ](* End If *)
   ,{i,Length@PosRep}];(* End Do *)
  
   (* Vector of permutations of proteins *)
   perm={Table[0,{i,Sites[[gene]]}]};
   tmp=Table[0,{i,Sites[[gene]]}];
   Do[
      tmp=ReplacePart[tmp,i->1];
      perm=Join[perm,Permutations[tmp]];
   ,{i,Length@tmp}];
 
   inter=Table[perm[[i]] Join[act1,rep1],{i,Length@perm}];

   Do[
      PAct=inter[[i,1;;ActSites[[gene]]]];
      PRep=inter[[i,ActSites[[gene]]+1;;Sites[[gene]]]];
      inter[[i]]={PAct,PRep};
   ,{i,Length@inter}];

   (* Vector of all the reactions: "reacs"
      Vector of the reactions per gene: "r" *)
   r={};
   Do[  
      Act0=Flatten@Position[inter[[k,1]],0];
      Rep0=Flatten@Position[inter[[k,2]],0];

      (* Reactions with activation proteins *)
      If[Act0!={},
         Do[
            X[[gene]]=inter[[k,1]];
            Y[[gene]]=inter[[k,2]];
            J=Act0[[j]];
            r1=Elem[X[[gene]],Y[[gene]]]+act1[[J]];
            X[[gene]]=ReplacePart[inter[[k,1]],act1[[J]],J];
            r2=Elem[X[[gene]],Y[[gene]]];
            r=Append[r,{r1,r2}];
         ,{j,Length@Act0}];
        ];

      (* Reactions with repression proteins *)
      If[Rep0!={},
         Do[
            X[[gene]]=inter[[k,1]];
            Y[[gene]]=inter[[k,2]];
            J=Rep0[[j]];
            r1=Elem[X[[gene]],Y[[gene]]]+rep1[[J]];
            Y[[gene]]=ReplacePart[inter[[k,2]],rep1[[J]],J];
            r2=Elem[X[[gene]],Y[[gene]]];
            r=Append[r,{r1,r2}];
         ,{j,Length@Rep0}];
        ];

     ,{k,Length@inter}];
   reacs=Append[reacs,r];
     
   (* Tensorial notation *)
   el1=Table[r[[i,1,2]],{i,Length@r}];
   el2=Table[r[[i,2]],{i,Length@r}];

   r1=Table[Subscript[Elem, Row[el1[[i,2]],","]]^Row[el1[[i,1]],","],{i,Length@r}];
   r2=Table[Subscript[Elem, Row[el2[[i,2]],","]]^Row[el2[[i,1]],","],{i,Length@r}];

   Do[
      r=ReplacePart[r,{{i,1,2}->r1[[i]]}];
      r=ReplacePart[r,{{i,2}->r2[[i]]}];
      r[[i]]=r[[i,1]] \[LeftRightArrow] r[[i,2]]
   ,{i,Length@r}];

   (* Total reactions: "RTot" *)
   ReactionsGeneProtein = Flatten@Append[ReactionsGeneProtein,r];


   (* Total parameters: "ParTot" *)
   ParametersFw=Flatten@Append[ParametersFw,Table[Subscript[Subscript[ToLowerCase[Elem],r[[i,1,2,1,2]]]^r[[i,1,2,2]], ToLowerCase[r[[i,1,1]]]],{i,Length@r}]];
   ParametersBw=Flatten@Append[ParametersBw,Table[Subscript[Subscript[ToLowerCase[Elem],r[[i,1,2,1,2]]]^r[[i,1,2,2]], -ToLowerCase[r[[i,1,1]]]],{i,Length@r}]];


   (* Production of proteins *)
   (* Vector of permutations of active sites *)
   perm={Table[0,{i,ActSites[[gene]]}]};
   tmp=Table[0,{i,ActSites[[gene]]}];
   Do[
      tmp=ReplacePart[tmp,i->1];
      perm=Join[perm,Permutations[tmp]];
   ,{i,Length@tmp}];

   interact=Table[perm[[i]] act1,{i,Length@perm}];
   interact=Drop[interact,1];


   perm={Table[0,{i,RepSites[[gene]]}]};
   tmp=Table[0,{i,RepSites[[gene]]}];
   Do[
      tmp=ReplacePart[tmp,i->1];
      perm=Join[perm,Permutations[tmp]];
   ,{i,Length@tmp}];

   interrep=Table[perm[[i]] rep1,{i,Length@perm}];
   interrep=Drop[interrep,1];


   sub=Table[0,{i,Length@rep1}];
   PrependTo[interrep,sub];

   Do[
      ReactionsProduction=Append[ReactionsProduction,Subscript[Elem,Row[interrep[[j]],","]]^Row[interact[[i]],","]->Subscript[Elem,Row[interrep[[j]],","]]^Row[interact[[i]],","]+Elem];
      ParametersProduction=Append[ParametersProduction, Subscript[Subscript[Global`p,Row[interrep[[j]],","]]^Row[interact[[i]],","], Elem]];
   ,{j,Length@interrep},{i,Length@interact}];

   (* Degradation of proteins *)
   If[Sites[[gene]]!=0,
      ReactionsDegradation=Append[ReactionsDegradation,Elem -> " "];
      ParametersDegradation=Append[ParametersDegradation, Subscript[Global`d, Elem]];
     ];
   
,{gene,Length@El}];

RTot=Join[ReactionsGeneProtein,ReactionsProduction,ReactionsDegradation];

(*NM*)
If[
   Length@ConstantProteins>0,
   ParTot=Join[ParametersFw,ParametersProduction,ParametersDegradation,ParametersBw,ConstantProteins],
   ParTot=Join[ParametersFw,ParametersProduction,ParametersDegradation,ParametersBw]
];
(*NM*)

Global`ProteinsWithoutGenes=ProteinsWithoutGenes;
(*NM*)
Global`ConstantProteins=ConstantProteins;
(*NM*)
Global`Sites=Sites;
Global`ReactionsGeneProtein=ReactionsGeneProtein;
Global`ParametersFw=ParametersFw;
Global`ParametersBw=ParametersBw;
Global`LengthParBw=Length@ParametersBw;
Global`ReactionsProduction=ReactionsProduction;
Global`ParametersProduction=ParametersProduction;
Global`ReactionsDegradation=ReactionsDegradation;
Global`ParametersDegradation=ParametersDegradation;
Global`El=El;
Global`reactions=RTot/. Table[El[[i]]->ToExpression@El[[i]],{i,Length@El}];
Global`parameters=ParTot/. Table[ToLowerCase@El[[i]]->ToExpression@ToLowerCase@El[[i]],{i,Length@El}];
Global`parameters=Global`parameters/. Table[El[[i]]->ToExpression@El[[i]],{i,Length@El}];

Return[{Global`reactions,Global`parameters}];
 
];


ReactionsOneSite[activation_, repression_]:=Block[{},

act=activation;
rep=repression;

p=Global`p;
d=Global`d;
El=Global`El;
graph=Global`graph;

Global`FirstTime=1;

(* Converts activations and repressions into strings *)
var=Union@Flatten@Table[{Variables@act[[i,1]],Variables@act[[i,2]]},{i,Length@act}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
act=act/.Table[var[[i]]->varString[[i]],{i,Length@var}];
var=Union@Flatten@Table[{Variables@rep[[i,1]],Variables@rep[[i,2]]},{i,Length@rep}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
rep=rep/.Table[var[[i]]->varString[[i]],{i,Length@var}];

(* Computes the parameters *)
ind1=Table[ToLowerCase[act[[i,1]]],{i,Length@act}];
ind2=Table[ToLowerCase[act[[i,2]]],{i,Length@act}];
actpar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@act}];
ind1=Table[ToLowerCase[rep[[i,1]]],{i,Length@rep}];
ind2=Table[ToLowerCase[rep[[i,2]]],{i,Length@rep}];
reppar=Table[Subscript[ind2[[i]], ind1[[i]]],{i,Length@rep}];
actw=Table[{act[[i]],actpar[[i]]},{i,Length@act}];
repw=Table[{rep[[i]],reppar[[i]]},{i,Length@rep}];
graph=Join[actw,repw];


El=VertexList[graph];	
Do[Symbolize[ ParsedBoxWrapper [ SubscriptBox[El[[i]],"_"] ] ],{i,Length@El}];
(* Do[Symbolize[ ParsedBoxWrapper [ SuperscriptBox[El[[i]],"_"] ] ],{i,Length@El}]; *)
 
(* Converts activations and repressions into strings *)
var=Union@Flatten@Table[{Variables@act[[i,1]],Variables@act[[i,2]]},{i,Length@act}];
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
act=act/.Table[var[[i]]->varString[[i]],{i,Length@var}];
var=Union@Flatten@Table[{Variables@rep[[i,1]],Variables@rep[[i,2]]},{i,Length@rep}];    
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
rep=rep/.Table[var[[i]]->varString[[i]],{i,Length@var}];

ActSites=Table[0,{Length[El]}];
Do[
   Do[
      If[El[[j]]==act[[i,2]],
         ActSites[[j]]++]
   ,{i,Length[act]}];
,{j,Length[El]}];

RepSites=Table[0,{Length[El]}];
Do[
   Do[
      If[El[[j]]==rep[[i,2]],
         RepSites[[j]]++]
   ,{i,Length[rep]}];
,{j,Length[El]}];

Sites=ActSites+RepSites;

           Print["Proteins = ",El];
   Print[];Print["Activation Sites per Gene = ",ActSites];
   Print[];Print["Repression Sites per Gene = ",RepSites];
   Print[];Print["Total Sites per Gene = ",Sites];

ProteinsWithoutGenesSites={};ProteinsWithoutGenes={};
Do[
   If[Sites[[i]]==0,
      ProteinsWithoutGenesSites=Append[ProteinsWithoutGenesSites,1];ProteinsWithoutGenes=Append[ProteinsWithoutGenes,El[[i]]],
      ProteinsWithoutGenesSites=Append[ProteinsWithoutGenesSites,0]
   ];
,{i,Length@Sites}];
Print[];Print["Proteins Without Genes = ", ProteinsWithoutGenes];

(*NM*)
ConstantProteins={};
Do[
   If[StringMatchQ[ProteinsWithoutGenes[[i]],__~~"$"],
      ConstantProteins=Append[ConstantProteins,ProteinsWithoutGenes[[i]]]
     ]
,{i,Length@ProteinsWithoutGenes}];
Print[];Print["Constant Proteins = ", ConstantProteins];
(*NM*)


(* Loop per gene *)
ReactionsGeneProtein={};
ParametersFw={};   
ParametersBw={};   
ReactionsProduction={};
ParametersProduction={};
ReactionsDegradation={};
ParametersDegradation={};
reacs={};
RTot={};
ParTot={};
Do[
   Elem=El[[gene]];
   PosAct=Position[act,Elem];
   PosRep=Position[rep,Elem];

   (* Activation proteins *)
   act1={};
   Do[
      If[PosAct[[i,2]]==2,
         act1=Append[act1,act[[PosAct[[i,1]],1]]];
        ]
   ,{i,Length@PosAct}];

   (* Repression proteins *)
   rep1={};
   Do[
      If[PosRep[[i,2]]==2,
         rep1=Append[rep1,rep[[PosRep[[i,1]],1]]];
        ]
   ,{i,Length@PosRep}];

   (* Tensorial notation *)
   rAct=Table[Subscript[Elem, 0]+act1[[i]] \[LeftRightArrow] Subscript[Elem, act1[[i]]],{i,Length@act1}];
   (* rAct=Table[Subscript[Elem, O]+act1[[i]] \[LeftRightArrow] Superscript[Elem, act1[[i]]],{i,Length@act1}]; *)
   rRep=Table[Subscript[Elem, 0]+rep1[[i]] \[LeftRightArrow] Subscript[Elem, rep1[[i]]],{i,Length@rep1}];
   r=Join[rAct,rRep];

   (* Total reactions: "RTot" *)
   ReactionsGeneProtein = Flatten@Append[ReactionsGeneProtein,r];

   (* Total parameters: "ParTot" *)
   ParametersFw=Flatten@Append[ParametersFw,Table[Subscript[ToLowerCase[Elem], ToLowerCase[r[[i,1,1]]]],{i,Length@r}]];
   ParametersBw=Flatten@Append[ParametersBw,Table[Subscript[ToLowerCase[Elem], -ToLowerCase[r[[i,1,1]]]],{i,Length@r}]];

   (* Production of proteins *)
   Do[
      ReactionsProduction=Append[ReactionsProduction,Subscript[Elem,act1[[i]]]->Subscript[Elem,act1[[i]]]+Elem];
      (* ReactionsProduction=Append[ReactionsProduction,Superscript[Elem,act1[[i]]]->Superscript[Elem,act1[[i]]]+Elem]; *)
      ParametersProduction=Append[ParametersProduction, Subscript[Global`p, Elem]];
   ,{i,Length@act1}];

   (* Degradation of proteins *)
   If[Sites[[gene]]!=0,
      ReactionsDegradation=Append[ReactionsDegradation,Elem -> " "];
      ParametersDegradation=Append[ParametersDegradation, Subscript[Global`d, Elem]]
     ];
,{gene,Length@El}];

RTot=Join[ReactionsGeneProtein,ReactionsProduction,ReactionsDegradation];
(*ParTot=Join[ParametersFw,ParametersProduction,ParametersDegradation,ParametersBw,ProteinsWithoutGenes];*)
(*ParTot=Join[ParametersFw,ParametersProduction,ParametersDegradation,ParametersBw];*)
   
(*NM*)
If[
   Length@ConstantProteins>0,
   ParTot=Join[ParametersFw,ParametersProduction,ParametersDegradation,ParametersBw,ConstantProteins],
   ParTot=Join[ParametersFw,ParametersProduction,ParametersDegradation,ParametersBw]
];
(*NM*)

Global`ProteinsWithoutGenes=ProteinsWithoutGenes;
(*NM*)
Global`ConstantProteins=ConstantProteins;
(*NM*)
Global`Sites=Sites;
Global`ReactionsGeneProtein=ReactionsGeneProtein;
Global`ParametersFw=ParametersFw;
Global`ParametersBw=ParametersBw;
Global`LengthParBw=Length@ParametersBw;
Global`ReactionsProduction=ReactionsProduction;
Global`ParametersProduction=ParametersProduction;
Global`ReactionsDegradation=ReactionsDegradation;
Global`ParametersDegradation=ParametersDegradation;
Global`El=El;
Global`reactions=RTot/. Table[El[[i]]->ToExpression@El[[i]],{i,Length@El}];
Global`parameters=ParTot/. Table[ToLowerCase@El[[i]]->ToExpression@ToLowerCase@El[[i]],{i,Length@El}];
Global`parameters=Global`parameters/. Table[El[[i]]->ToExpression@El[[i]],{i,Length@El}];

Return[{Global`reactions,Global`parameters}];

]


SubstanceNames[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
For[i=1,i<=m,i++,{If[var[[i]]==" ",{var=Drop[var,i];m--}]}];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(*NM*)
For[i=1,i<=m,i++,{If[MemberQ[ConstantProteins,ToString@var[[i]]],{var=Drop[var,{i}];m--;i--}]}];
(*NM*)
(* Constant proteins are dropped *)

output=var;
Global`substanceNames=output; 
Return[output];

]; (* End of SubstanceNames[reactions] *)


SubstanceVariables[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
For[i=1,i<=m,i++,{If[var[[i]]==" ",{var=Drop[var,i];m--}]}];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

For[i=1,i<=m,i++,{If[MemberQ[ConstantProteins,ToString@var[[i]]],{var=Drop[var,{i}];m--;i--}]}];
(* Constant proteins are dropped *)

output=var;
(*careful*)
Do[
If[MemberQ[Global`ProteinsWithoutGenes,ToString@output[[i]]]==False || 1==1,
   output[[i]]=output[[i]]/.output[[i]]->output[[i]][Global`t];
];
,{i,m}];

Global`substanceVariables=output;
Return[output];

]; (* End of SubstanceVariables[reactions] *)


SubstanceInitialConditions[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
For[i=1,i<=m,i++,{If[var[[i]]==" ",{var=Drop[var,i];m--}]}];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)


For[i=1,i<=m,i++,{If[MemberQ[ConstantProteins,ToString@var[[i]]],{var=Drop[var,{i}];m--;i--}]}];
(* Constant proteins are dropped *)

output=var;

(*careful*)
Do[
If[MemberQ[Global`ProteinsWithoutGenes,ToString@output[[i]]]==False || 1==1,
   output[[i]]=output[[i]]/.output[[i]]->output[[i]][0]== Global`InitCond[i],
   output[[i]]=output[[i]]/.output[[i]]->output[[i]]== Global`InitCond[i]
];
,{i,m}];


Global`substanceInitialConditions=output;
Return[output];

]; (* End of SubstanceInitialConditions[reactions] *)


ParameterNames[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction, parametros,paraux,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
   If[StringMatchQ[ToString[reacs[[i]]],"*\[LeftRightArrow]*"],
      doubleReaction=Append[doubleReaction,1],doubleReaction=Append[doubleReaction,0]
     ];(* End If *)
,{i,n}];(* End Do *)

reacs=Flatten@Table[If[doubleReaction[[i]]== 0,reacs[[i]],{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}],{i,n}];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)

(* Parameters ki of all the reactions *)
parametros=Table[Subscript[Global`k, i] ,{i,n}] ;
paraux={};
Do[If[doubleReaction[[i]]== 1,paraux=Append[paraux,Subscript[Global`k, -i]]],{i,n}];

Global`LengthParBw=Length@paraux;

parametros=Join[parametros,paraux];
output=Flatten@Append[parametros,ProteinsWithoutGenes];
Return[output];

];(* End of ParameterNames[reactions] *)


ParameterInput[parameters_]:=
Block[{par,n,var,i,m,m1,doubleReaction, parametros,paraux,isSteady,tmp,j,output},

par=parameters;

Do[ par[[i]]= par[[i]] -> Global`ParInputVar[i],{i,Length[par]}];
output=par;

Global`parameterInput=output;
Return[output];
];(* End of ParameterInput[reactions] *)


ReactionGraph[reactions_,parameters_]:=
Block[{reacs,par,n,var,i,m,doubleReaction,ta,nfw,d},

reacs=reactions;
par=parameters;

d=Global`d;
p=Global`p;

n=Length[reacs];

If[Length@ConstantProteins > 0,
nfw=Length@par-Length@ProteinsWithoutGenes-Global`LengthParBw,
nfw=Length@par-Global`LengthParBw;
];

var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
   If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
      doubleReaction=Append[doubleReaction,1],
      doubleReaction=Append[doubleReaction,0]
     ];(* End If *)
,{i,n}];(*End Do*)

ta=Table[
         If[doubleReaction[[i]]==0,
            DisplayForm@OverscriptBox["\[LongRightArrow]",Subscript@par[[i]]],
            DisplayForm@UnderoverscriptBox["\[RightArrowLeftArrow]",Subscript@par[[i+nfw]],Subscript@par[[i]]]
           ](*End If*)
   ,{i,n}];(*End Table*)

Do[ Print[reactions[[i,1]]," ",ta[[i]]," ",reactions[[i,2]]],{i,n}];

]; (* End of ReactionGraph[reactions] *)


Equations[reactions_,parameters_]:=
Block[{reacs,par,n,nfw,var,i,m,m1,doubleReaction,ta,parametros,isSteady,varString,constant,tmp,j,Alpha,Beta,Gamma,Omega,output},

reacs=reactions;
par=parameters;


(*NM*)
If[Length@ConstantProteins>0,
   par=Drop[par,-Length@ConstantProteins]
];
(*NM*)

n=Length[reacs];
nfw=n-Global`LengthParBw;
 
(* Introduces "Garbage" variables in the degradations *)
j=1;
Do[
   If[reacs[[i,2]]==" ",
      reacs[[i,2]]=Subscript[Global`G, j];j++
     ];(* End If *)
,{i,Length@reacs}];(* End Do *)

var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];

(* Appends $ to constant variables *)
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
reacs=reacs/.Table[var[[i]]->varString[[i]],{i,Length@var}];


Do[
   If[MemberQ[Global`ConstantProteins,ToString@varString[[i]]],
      constant=ToExpression@StringJoin[ToString@varString[[i]],"\[Sterling]"];reacs=reacs/.varString[[i]]->constant
     ]
,{i,Length@varString}];
reacs=reacs/.Table[varString[[i]]->var[[i]],{i,Length@var}];


var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
   If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
      doubleReaction=Append[doubleReaction,1],
      doubleReaction=Append[doubleReaction,0]
     ];(* End If *)
,{i,n}];(* End Do *)

reacs=Flatten@Table[If[doubleReaction[[i]]== 0,reacs[[i]],{reacs[[i,1]]-> reacs[[i,2]]}],{i,n}];

Do[
   If[doubleReaction[[i]]== 1,
      reacs=Flatten@Append[reacs,reacs[[i,2]]-> reacs[[i,1]]];
     ];(* End If *)
,{i,n}];(* End Do *)
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,
    {If[StringMatchQ[ToString[var[[i]]],"*\[Sterling]"],
        (**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
        reacs=reacs/.var[[i]]->tmp;
        var[[i]]=tmp;
        Do[
           If[var[[i]]==var[[j]],
              (**If found**){isSteady[[j]]=True;
              var=Drop[var,{i}];
              m--;
              i--;
              Goto[fim];}
             ];(* End If *)
        ,{j,i-1,1,-1}];(* End Do *)
        isSteady=Append[isSteady,True];},
        (* If False *) isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

(* Mass Action*)
Do[
   {Alpha[i,j]=Coefficient[reacs[[i,1]],var[[j]]],Beta[i,j]=Coefficient[reacs[[i,2]],var[[j]]]}
,{i,n},{j,m}];(**End Do**)

Gamma=Table[Beta[i,j]-Alpha[i,j],{i,n},{j,m}];

(*Print[par];*)

Do[
   Omega[i]=par[[i]]*Product[var[[j]]^Alpha[i,j],{j,m}]
,{i,n}];

output=tmp={};

Do[
   If[isSteady[[j]],
      tmp=Append[tmp,var[[j]][Global`t]->var[[j]]],
      output=Append[output,D[var[[j]][Global`t],Global`t]==
      Sum[Gamma[[i,j]]*par[[i]]*Product[var[[j]][Global`t]^Alpha[i,j],{j,m}],{i,n}]]
     ];(* End If *)
,{j,m}];(** End Do **) 
output=output/.tmp;

Global`equations=output;

(* If KineticEquations has been run for the first time after Reactions, the notebook "Equation" will be open to enable Symbolize to be effective *)
If[Global`FirstTime==1,CreateNB[]; Global`FirstTime++];

Return[output];
]; (* End of KineticEquations[reactions,parameters] *)


ConservationLaws[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,parametros,isSteady,varString,constant,tmp,j,Alpha,Beta,Gamma,Omega,Sigma,output},

reacs=reactions;

n=Length[reacs];

(* Introduces "Garbage" variables in the degradations *)
j=1;
Do[
   If[reacs[[i,2]]==" ",
      reacs[[i,2]]=Subscript[Global`G, j];j++
     ];(*End If*)
,{i,Length@reacs}];(*End Do*)

var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];


(* Appends \[Sterling] to constant proteins in reacs *)
varString=var/.Table[var[[i]]->ToString@var[[i]],{i,Length@var}];
reacs=reacs/.Table[var[[i]]->varString[[i]],{i,Length@var}];
Do[
   If[MemberQ[Global`ProteinsWithoutGenes,ToString@varString[[i]]],
      constant=ToExpression@StringJoin[ToString@varString[[i]],"\[Sterling]"];
      reacs=reacs/.varString[[i]]->constant;
     ];
,{i,Length@varString}];
reacs=reacs/.Table[varString[[i]]->var[[i]],{i,Length@var}];



var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *\[Sterling]  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
   If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
      doubleReaction=Append[doubleReaction,1],
      doubleReaction=Append[doubleReaction,0]
     ];(* End If *)
,{i,n}];(* End Do *)

reacs=Flatten@Table[If[doubleReaction[[i]]== 0,reacs[[i]],{reacs[[i,1]]-> reacs[[i,2]]}],{i,n}];

Do[
   If[doubleReaction[[i]]== 1,
      reacs=Flatten@Append[reacs,reacs[[i,2]]-> reacs[[i,1]]];
     ];(* End If *)
,{i,n}];(* End Do *)

(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,
    {If[StringMatchQ[ToString[var[[i]]],"*\[Sterling]"],
        (**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
        reacs=reacs/.var[[i]]->tmp;
        var[[i]]=tmp;
        Do[
           If[var[[i]]==var[[j]],
              (**If found**){isSteady[[j]]=True;
              var=Drop[var,{i}];
              m--;
              i--;
              Goto[fim];}
             ];(* End If *)
        ,{j,i-1,1,-1}];(**End Do**)
        isSteady=Append[isSteady,True]},
        (**If False**)isSteady=Append[isSteady,False]
       ];(* End If *)
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

(* Mass Action*)
Do[
   {Alpha[i,j]=Coefficient[reacs[[i,1]],var[[j]]],Beta[i,j]=Coefficient[reacs[[i,2]],var[[j]]]}
,{i,n},{j,m}];(**End Do**)

Gamma=Table[Beta[i,j]-Alpha[i,j],{i,n},{j,m}];

Do[
   If[isSteady[[j]],
      Do[Gamma[[i,j]]=0,{i,n}] 
     ];(* End If *)
,{j,m}];(* End Do *)
Sigma=NullSpace[Gamma];

output=tmp={};
Do[output=Append[output,Sum[Sigma[[j,i]]var[[i]][Global`t],{i,m}]],{j,Length[Sigma]}];
Do[If[isSteady[[j]],tmp=Append[tmp,var[[j]][Global`t]->var[[j]]]],{j,m}];

output=output/.tmp;
Return[output];
]; (* End of ConservationLaws[reactions] *)


EulerIntegration[equations_, parameters_, initialConditions_, variables_, iterations_, step_]:=
Block[{i, j, eq, par, inCond, iter, h, var, T, Variable, iterVar, eqValue, output},

eq = equations;
par = parameters;
inCond = initialConditions;
iter = iterations;
h = step;
var = variables;

T = Table[Null,{i,iter+1}];
T[[1]] = 0.;
Do[var[[i]]=var[[i]]->Variable[i],{i,Length@var}];
Do[Variable[i]=inCond[[i,2]],{i,Length@var}];
iterVar=Table[Null,{i,iter+1},{j,Length@var}];
Do[iterVar[[1,i]]=Variable[i],{i,Length@var}];


Do[
   eqValue=eq[[All,2]]/.par/.var;
   Do[
      var[[i,2]]=var[[i,2]]+h eqValue[[i]];
      iterVar[[j+1,i]]=var[[i,2]]
      ,{i,Length@var}];
   T[[j+1]]=T[[j]]+h;
,{j,iter}];

output = Table[{T[[i]],iterVar[[i,j]]},{j,Length@var},{i,Length@T}];

Return[output];

]; (* End of EulerIntegration *)


EulerPoints[equations_, parameters_, initialConditions_, variables_, iterations_, step_]:=
Block[{i, j, eq, par, inCond, iter, h, var, t, T, Variable, iterVar, eqValue, eqValueTemp, output},

eq = equations;
par = parameters;
inCond = initialConditions;
iter = iterations;
h = step;
var = variables;


Do[var[[i]]=var[[i]]->Variable[i],{i,Length@var}];
Do[Variable[i]=inCond[[i,2]],{i,Length@var}];


eqValueTemp = eq[[All,2]]/.par;

Do[
   eqValue=eqValueTemp/.var;
   Do[
      var[[i,2]]=var[[i,2]]+h eqValue[[i]];
      ,{i,Length@var}];
,{j,iter}];

(*
Do[
   eqValue=eq[[All,2]]/.par/.var;
   Do[
      var[[i,2]]=var[[i,2]]+h eqValue[[i]];
      ,{i,Length@var}];
,{j,iter}];
*)


output = var[[All,2]];

Return[output];

]; (* End of EulerPoints *)


ParametersWithoutRepetition[parameters_]:=
Block[{length1, length2, length3, length4, start, nrepetitions, found, parametersWithoutRepetitions, parametersFwWithoutRepetitions, parametersProductionWithoutRepetitions, 
parametersDegradationWithoutRepetitions, parametersBwWithoutRepetitions, parametersWithoutRepetitionsPosition, parametersFwWithoutRepetitionsPosition, 
parametersProductionWithoutRepetitionsPosition, parametersDegradationWithoutRepetitionsPosition, parametersBwWithoutRepetitionsPosition},

parametersWithoutRepetitions={};
parametersFwWithoutRepetitions={};
parametersProductionWithoutRepetitions={};
parametersDegradationWithoutRepetitions={};
parametersBwWithoutRepetitions={};

length1 = Length@Global`ParametersFw;
length2 = Length@Global`ParametersFw + Length@Global`ParametersProduction;
length3 = Length@Global`ParametersFw + Length@Global`ParametersProduction + Length@Global`ParametersDegradation;
length4 = Length@Global`ParametersFw + Length@Global`ParametersProduction + Length@Global`ParametersDegradation + Length@Global`ParametersBw;

Do[nrepetitions=0;
   Do[
     If[(MatchQ[parameters[[i,1]],parameters[[j,1]]] && MatchQ[parameters[[i,2]],parameters[[j,2]]]),
     nrepetitions++];
   ,{j,1,i-1}];
   If[
      nrepetitions==0,
      parametersWithoutRepetitions=Append[parametersWithoutRepetitions,parameters[[i]]];
      If[i <= length1,
         parametersFwWithoutRepetitions=Append[parametersFwWithoutRepetitions,parameters[[i]]];
      ];
      If[i > length1 && i <= length2,
         parametersProductionWithoutRepetitions=Append[parametersProductionWithoutRepetitions,parameters[[i]]];
      ];
      If[i > length2 && i <= length3,
         parametersDegradationWithoutRepetitions=Append[parametersDegradationWithoutRepetitions,parameters[[i]]];
      ];
      If[i > length3 && i <= length4,
         parametersBwWithoutRepetitions=Append[parametersBwWithoutRepetitions,parameters[[i]]];
      ];
   ];
,{i,1,Length@parameters-Length@Global`ConstantProteins}];


parametersWithoutRepetitionsPosition=Table[0,{i,Length@parametersWithoutRepetitions}];
parametersFwWithoutRepetitionsPosition=Table[0,{i,Length@parametersFwWithoutRepetitions}];
parametersProductionWithoutRepetitionsPosition=Table[0,{i,Length@parametersProductionWithoutRepetitions}];
parametersDegradationWithoutRepetitionsPosition=Table[0,{i,Length@parametersDegradationWithoutRepetitions}];
parametersBwWithoutRepetitionsPosition=Table[0,{i,Length@parametersBwWithoutRepetitions}];

Do[
   found=False;
   Do[
      If[
         parametersWithoutRepetitions[[i]]==Global`parameterInput[[j,1]] && !found,
         found=True;
         parametersWithoutRepetitionsPosition[[i]]= j;
         If[j <= length1,
            parametersFwWithoutRepetitionsPosition[[i]] = j;
         ];
         If[j > length1 && j <= length2,
            If[j == length1 +1, start=i-1];
            parametersProductionWithoutRepetitionsPosition[[i-start]] = j;
         ];
         If[j > length2 && j <= length3,
            If[j == length2 +1, start=i-1];
            parametersDegradationWithoutRepetitionsPosition[[i-start]] = j;
         ];
         If[j > length3 && j <= length4,
            If[j == length3 +1, start=i-1];
            parametersBwWithoutRepetitionsPosition[[i-start]] = j;
         ];
      ];
   ,{j,1,Length@Global`parameterInput}];
,{i,1,Length@parametersWithoutRepetitions}];


Global`parametersWithoutRepetitions = parametersWithoutRepetitions;
Global`parametersFwWithoutRepetitions = parametersFwWithoutRepetitions;
Global`parametersProductionWithoutRepetitions = parametersProductionWithoutRepetitions;
Global`parametersDegradationWithoutRepetitions = parametersDegradationWithoutRepetitions;
Global`parametersBwWithoutRepetitions = parametersBwWithoutRepetitions;

Global`parametersWithoutRepetitionsPosition = parametersWithoutRepetitionsPosition;
Global`parametersFwWithoutRepetitionsPosition = parametersFwWithoutRepetitionsPosition;
Global`parametersProductionWithoutRepetitionsPosition = parametersProductionWithoutRepetitionsPosition;
Global`parametersDegradationWithoutRepetitionsPosition = parametersDegradationWithoutRepetitionsPosition;
Global`parametersBwWithoutRepetitionsPosition = parametersBwWithoutRepetitionsPosition;

Return[parametersWithoutRepetitions];

]; (* End of ParametersWithoutRepetition[parameters_] *)


EmptyGenesPosition[initialConditions_, emptyGenes_]:=
Block[{emptyGenesPosition},

emptyGenesPosition=Table[0,{i,Length@emptyGenes}];
Do[
   Do[
      If[initialConditions[[j,1]]==emptyGenes[[i]][0],
      emptyGenesPosition[[i]]= j;
   ];
   ,{j,Length@initialConditions}];
,{i,Length@emptyGenes}];

Global`emptyGenesPosition = emptyGenesPosition;

Return[emptyGenesPosition];

]; (* End of EmptyGenesPosition *)


CEquations[equations_, substanceVariables_, parameters_]:=
Block[{variablesMathToC, parametersMathToC, eq, s, initialString, str, st, string, finalString, parametersWithoutRepetitions, nrepetitions},


ParametersWithoutRepetition[Global`parameters];

variablesMathToC =Table[0,{i,Length@substanceVariables}];
Do[variablesMathToC[[i]]=substanceVariables[[i]]->Global`v[i-1],{i,Length@substanceVariables}];



parametersWithoutRepetitions={};
parametersWithoutRepetitions=Append[parametersWithoutRepetitions,parameters[[1]]];
Do[nrepetitions=0;
   Do[
     If[(MatchQ[parameters[[i,1]],parameters[[j,1]]] && MatchQ[parameters[[i,2]],parameters[[j,2]]]),
     nrepetitions++];
   ,{j,1,i-1}];
   If[nrepetitions==0,parametersWithoutRepetitions=Append[parametersWithoutRepetitions,parameters[[i]]]];
,{i,2,Length@parameters-Length@Global`ProteinsWithoutGenes}];



Do[
  parametersWithoutRepetitions=Append[parametersWithoutRepetitions,parameters[[i]]];
,{i,Length@parameters-Length@Global`ProteinsWithoutGenes+1,Length@parameters}];
(*parametersWithoutRepetitions = Join[parametersWithoutRepetitions,Global`ProteinsWithoutGenes];*)


parametersMathToC =Table[0,{i,Length@parametersWithoutRepetitions}];
Do[parametersMathToC[[i]]=parametersWithoutRepetitions[[i]]->Global`p[i-1],{i,Length@parametersWithoutRepetitions}];

eq=equations[[All,2]]/.variablesMathToC/.parametersMathToC;

s=OpenWrite["equations.h"];
(*s=OpenWrite["/home/daniele/mathematica/NewModel/equations.h"];*)

initialString = "void Equations(double *v, double *p, double *F)\n{\n";
WriteString[s,initialString];

str=Table[Null,{i,Length@eq}];
Do[
st=Characters@ToString@eq[[i]];
string=st;
Do[
If[st[[i]]==" ",
If[st[[i-1]]!="-" && st[[i+1]]!="-" &&
st[[i-1]]!="+" && st[[i+1]]!="+" ,

  string[[i]]=st[[i]]/." "->"*",	

  string[[i]]= " "];
];
,{i,2,Length@st -1}];
string=StringJoin@string;

str[[i]]=StringJoin[{"  ",ToString@Global`F[i-1]," = ",string,";\n"}];
WriteString[s,str[[i]]];
,{i,Length@eq}];

finalString = "}\n";
WriteString[s,finalString];

FilePrint["equations.h"];

Close[s];

Global`parametersWithoutRepetitions = parametersWithoutRepetitions; 
]; (* End of CEquations *)


WriteDataC[]:=
Block[{s, str, string, NAct, NRep,parametersProductionWithoutRepetitionsO},

s=OpenWrite["data.txt"];
(*s=OpenWrite["/home/daniele/mathematica/NewModel/data.txt"];*)

ParametersWithoutRepetition[Global`parameters];

string = "# Vectors Lengths Variables ParametersRNA ParametersEmptyGenes ParametersFw ParametersProduction ParametersDegradation ParametersBw ProportionalityConstants\n";
WriteString[s,string];

str=Table[Null,{i,8}];
str[[1]]=StringJoin[{ToString[Length@Global`substanceVariables]," "}];
WriteString[s,str[[1]]];
str[[2]]=StringJoin[{ToString[Length@Global`a + Length@Global`L]," "}];
WriteString[s,str[[2]]];
str[[3]]=StringJoin[{ToString[Length@Global`emptyGenesValue]," "}];
WriteString[s,str[[3]]];
str[[4]]=StringJoin[{ToString[Length@Global`parametersFwValue]," "}];
WriteString[s,str[[4]]];
str[[5]]=StringJoin[{ToString[Length@Global`parametersProductionValue]," "}];
WriteString[s,str[[5]]];
str[[6]]=StringJoin[{ToString[Length@Global`parametersDegradationValue]," "}];
WriteString[s,str[[6]]];
str[[7]]=StringJoin[{ToString[Length@Global`parametersBwValue]," "}];
WriteString[s,str[[7]]];
str[[8]]=StringJoin[{ToString[Length@Global`parametersPropConst]," "}];
WriteString[s,str[[8]]];

WriteString[s,"\n\n"];


(* string = "# a1 a2 a3 a4 a5 a6 a7 a8 a9 a10 L1 L2 L3 L4 L5 L6 L7 L8 L9 L10\n"; *)
string = "# a1 a2 a3 a4 L1 L2 L3 L4\n";
WriteString[s,string];

str=Table[Null,{i,Length@Global`a}];

Do[
str[[i]]=StringJoin[{ToString[Global`a[[i]]//N]," "}];

WriteString[s,str[[i]]];
,{i,Length@Global`a}];

str=Table[Null,{i,Length@Global`L}];
Do[
str[[i]]=StringJoin[{ToString[Global`L[[i]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`L}];
WriteString[s,"\n\n"];



str=Table[Null,{i,Length@Global`emptyGenesNotSymbolized}];
string=StringJoin[{"# Empty Genes"," "}];
WriteString[s,string];
Do[
str[[i]]=StringJoin[{ToString[Global`emptyGenesNotSymbolized[[i,1,1]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`emptyGenesNotSymbolized}];
string=StringJoin[{"Value Position"," "}];
WriteString[s,string];
WriteString[s,"\n"];

str=Table[Null,{i,Length@Global`emptyGenesValue}];
Do[
str[[i]]=StringJoin[{ToString[Global`emptyGenesValue[[i]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`emptyGenesValue}];
WriteString[s,"\n"];

str=Table[Null,{i,Length@Global`emptyGenesPosition}];
Do[
str[[i]]=StringJoin[{ToString[Global`emptyGenesPosition[[i]]]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`emptyGenesPosition}];
WriteString[s,"\n\n"];


Global`parametersFwWithoutRepetitionsO = Global`parametersFwWithoutRepetitions/.{0->O};

str=Table[Null,{i,Length@Global`parametersFwWithoutRepetitionsO}];
string=StringJoin[{"#"," "}];
WriteString[s,string];
Do[
str[[i]]=StringJoin[ToString@Global`parametersFwWithoutRepetitionsO[[i,1,1,1]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersFwWithoutRepetitionsO[[i,1,2]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersFwWithoutRepetitionsO[[i,1,1,2]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersFwWithoutRepetitionsO[[i,2]]]; 
str[[i]]=StringJoin[str[[i]]," "]; 
WriteString[s,str[[i]]];
,{i,Length@Global`parametersFwWithoutRepetitionsO}];
WriteString[s,"\n"];

str=Table[Null,{i,Length@Global`parametersFwValue}];
Do[
str[[i]]=StringJoin[{ToString[Global`parametersFwValue[[i]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`parametersFwValue}];
WriteString[s,"\n\n"];


str=Table[Null,{i,Length@Global`parametersProductionWithoutRepetitions}];
string=StringJoin[{"#"," "}];
WriteString[s,string];

Print["\n"];

Do[

(*
Print[Global`parametersProductionWithoutRepetitions[[i,1,1,1]]];
Print[Global`parametersProductionWithoutRepetitions[[i,2]]];
*)

Global`parametersProductionWithoutRepetitionsO = Global`parametersProductionWithoutRepetitions/.{0->O};
str[[i]]=StringJoin[ToString@Global`parametersProductionWithoutRepetitionsO[[i,1,1,1]],ToString@Global`parametersProductionWithoutRepetitionsO[[i,2]]];  
NAct = Length@Global`parametersProductionWithoutRepetitions[[i,1,2,1]];
Do[
   str[[i]]=StringJoin[str[[i]],"_"]; 
   str[[i]]=StringJoin[str[[i]],ToString@Global`parametersProductionWithoutRepetitionsO[[i,1,2,1,j]]];
(*Print[ToLowerCase@ToString@Global`parametersProductionWithoutRepetitions[[i,2,1,1,2,1,j]]];*)
,{j,NAct}];
(*str[[i]]=StringJoin[str[[i]],ToLowerCase@ToString@Global`parametersProductionWithoutRepetitions[[i,1,2,1,1]]]; *)
(*
Print[Global`parametersProductionWithoutRepetitions[[i,1,1,2,1,1]]];
Print[Global`parametersProductionWithoutRepetitions[[i,1,1,2,1,2]]];
Print["\n"];
*)
NRep = Length@Global`parametersProductionWithoutRepetitionsO[[i,1,1,2,1]];
Do[
   str[[i]]=StringJoin[str[[i]],"_"]; 
   str[[i]]=StringJoin[str[[i]],ToString@Global`parametersProductionWithoutRepetitionsO[[i,1,1,2,1,j]]];
(*Print[ToLowerCase@ToString@Global`parametersProductionWithoutRepetitions[[i,2,1,1,2,1,j]]];*)
,{j,NRep}];

str[[i]]=StringJoin[str[[i]]," "]; 
WriteString[s,str[[i]]];
,{i,Length@Global`parametersProductionWithoutRepetitions}];
WriteString[s,"\n"];

str=Table[Null,{i,Length@Global`parametersProductionValue}];
Do[
str[[i]]=StringJoin[{ToString[Global`parametersProductionValue[[i]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`parametersProductionValue}];
WriteString[s,"\n\n"];



str=Table[Null,{i,Length@Global`parametersDegradationWithoutRepetitions}];
string=StringJoin[{"#"," "}];
WriteString[s,string];
Do[
str[[i]]=StringJoin[ToString@Global`parametersDegradationWithoutRepetitions[[i,1]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersDegradationWithoutRepetitions[[i,2]]]; 
str[[i]]=StringJoin[str[[i]]," "]; 
WriteString[s,str[[i]]];
,{i,Length@Global`parametersDegradationWithoutRepetitions}];
WriteString[s,"\n"];

str=Table[Null,{i,Length@Global`parametersDegradationValue}];
Do[
str[[i]]=StringJoin[{ToString[Global`parametersDegradationValue[[i]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`parametersDegradationValue}];
WriteString[s,"\n\n"];




Global`parametersBwWithoutRepetitionsO = Global`parametersBwWithoutRepetitions/.{0->O};

str=Table[Null,{i,Length@Global`parametersBwWithoutRepetitionsO}];
string=StringJoin[{"#"," "}];
WriteString[s,string];
Do[
str[[i]]=StringJoin[ToString@Global`parametersBwWithoutRepetitionsO[[i,1,1,1]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersBwWithoutRepetitionsO[[i,1,2]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersBwWithoutRepetitionsO[[i,1,1,2]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersBwWithoutRepetitionsO[[i,2]]]; 
str[[i]]=StringJoin[str[[i]]," "]; 
WriteString[s,str[[i]]];
,{i,Length@Global`parametersBwWithoutRepetitionsO}];
WriteString[s,"\n"];


(*
str=Table[Null,{i,Length@Global`parametersBwWithoutRepetitionsO}];
string=StringJoin[{"#"," "}];
WriteString[s,string];
Do[
str[[i]]=StringJoin[ToString@Global`parametersBwWithoutRepetitions[[i,1]],"_"]; 
str[[i]]=StringJoin[str[[i]],ToString@Global`parametersBwWithoutRepetitions[[i,2]]]; 
str[[i]]=StringJoin[str[[i]]," "]; 
WriteString[s,str[[i]]];
,{i,Length@Global`parametersBwWithoutRepetitionsO}];
WriteString[s,"\n"];
*)

str=Table[Null,{i,Length@Global`parametersBwValue}];
Do[
str[[i]]=StringJoin[{ToString[Global`parametersBwValue[[i]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`parametersBwValue}];
WriteString[s,"\n\n"];


(*
str=Table[Null,{i,Length@Global`El}];
string=StringJoin[{"# Proportionality Constants"," "}];
WriteString[s,string];
Do[
str[[i]]=StringJoin[{ToString[Global`substanceNames[[i]]]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`El-Length@Global`ProteinsWithoutGenes}];
Do[
str[[i]]=StringJoin[{ToString[Global`ProteinsWithoutGenes[[i]]]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`ProteinsWithoutGenes}];
WriteString[s,"\n"];
*)

str=Table[Null,{i,Length@Global`El}];
string=StringJoin[{"# Proportionality Constants"," "}];
WriteString[s,string];
Do[
str[[i]]=StringJoin[{ToString[Global`substanceNames[[i]]]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`El-Length@ConstantProteins}];
WriteString[s,"\n"];


str=Table[Null,{i,Length@Global`El}];
Do[
str[[i]]=StringJoin[{ToString[Global`parametersPropConst[[i]]//N]," "}];
WriteString[s,str[[i]]];
,{i,Length@Global`parametersPropConst}];
WriteString[s,"\n\n"];


string=StringJoin[{"#"," Time"}];
WriteString[s,string];
WriteString[s,"\n"];
str=ToString[Global`Time//N]; 
WriteString[s,str];
WriteString[s,"\n"];


FilePrint["data.txt"];

Close[s];

]; (* End of WriteDataC *)


CreateNB[]:=Block[{},

nb=CreateDocument[{
Cell["Substances Names","Text"],
ExpressionCell[Global`substanceNames,"Input",Evaluator->"Local"],
Cell["substanceNames=%","Input"],
Cell["Substances Variables","Text"],
ExpressionCell[Global`substanceVariables,"Input",Evaluator->"Local"],
Cell["substanceVariables=%","Input"],
Cell["Substances Initial Conditions","Text"],
ExpressionCell[Global`substanceInitialConditions,"Input",Evaluator->"Local"],
Cell["substanceInitialConditions=%","Input"],
Cell["Equations","Text"],
ExpressionCell[Global`equations,"Input",Evaluator->"Local"],
Cell["equations=%","Input"],
Cell["EmptyGenes","Text"],
ExpressionCell[Global`emptyGenes,"Input",Evaluator->"Local"],
Cell["emptyGenes=%","Input"]
},NotebookFileName->"tmp.txt",Visible->False];

SelectionMove[nb,Next,Cell,2];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,1];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,2];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,1];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,2];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,1];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,2];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,1];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,2];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,1];SelectionEvaluate[nb];

]
(* End of CreateNB[] *)


FillParameters[start_,finish_]:=
Block[{},

parameterInput=Global`parameterInput;
Do[i=j;
   While[If[MemberQ[ProteinsWithoutGenes,ToString@parameterInput[[i,1]]],Break[],parameterInput[[i,1,1]] == parameterInput[[j,1,1]]],
         If[parameterInput[[i,1]]==parameterInput[[j,1]],Global`ParInputVar[i]=Global`ParInputVar[j]];i++]
,{j,start,finish}];

]


EmptyGenes[value_]:=
Block[{},

Global`emptyInitialConditions=Global`emptyGenes/.Table[Global`emptyGenes[[i]]->Global`emptyGenes[[i]][0],{i,Length@Global`emptyGenes}];

nb=CreateDocument[{
Cell["Empty Initial Conditions","Text"],
ExpressionCell[Global`emptyInitialConditions,"Input",Evaluator->"Local"],
Cell["emptyInitialConditions=%","Input"]
},NotebookFileName->"tmp2.txt",Visible->False];
SelectionMove[nb,Next,Cell,2];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,1];SelectionEvaluate[nb];

Do[
   If[MemberQ[Global`emptyInitialConditions,Global`substanceInitialConditions[[i,1]]],
Global`InitCond[i]=value;
   ];
,{i,Length@Global`substanceInitialConditions}];

Global`emptyGenesValue=Table[0.,{i,Length@Global`emptyGenes}];
Do[
Global`emptyGenesValue[[i]] = value;
,{i,Length@Global`emptyGenes}];

]


NotEmptyGenes[value_]:=
Block[{},

notProteinsWithoutGenes=Complement[Global`El,Global`ProteinsWithoutGenes];
Global`emptyInitialConditions=Global`emptyGenes/.Table[Global`emptyGenes[[i]]->Global`emptyGenes[[i]][0],{i,Length@Global`emptyGenes}];

nb=CreateDocument[{
Cell["Empty Initial Conditions","Text"],
ExpressionCell[Global`emptyInitialConditions,"Input",Evaluator->"Local"],
Cell["emptyInitialConditions=%","Input"]
},NotebookFileName->"EmptyGenes",Visible->False];
SelectionMove[nb,Next,Cell,2];SelectionEvaluate[nb];
SelectionMove[nb,Next,Cell,1];SelectionEvaluate[nb];


Do[
   If[!MemberQ[Global`emptyInitialConditions,Global`substanceInitialConditions[[i,1]]] && !MemberQ[ToExpression@Global`El,Global`substanceInitialConditions[[i,1]]],
Global`InitCond[i]=value;
   ];
,{i,Length@notProteinsWithoutGenes +1,Length@Global`substanceInitialConditions}];

]


(* KINETICS *)


EquationsKinetics[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,ta,parametros,isSteady,tmp,j,Alpha,Beta,Gamma,Omega,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
doubleReaction=Append[doubleReaction,1],
doubleReaction=Append[doubleReaction,0]],{i,n}];(*End Do*)
ta=Table[If[doubleReaction[[i]]==0,
DisplayForm@OverscriptBox["\[LongRightArrow]",Subscript[Global`k, i]],
DisplayForm@UnderoverscriptBox["\[RightArrowLeftArrow]",Subscript[Global`k, -i],Subscript[Global`k, i]]],
{i,n}];(**End Table**)
m1=Sum[doubleReaction[[i]],{i,n}]; (* Number of constant variables *)
reacs=Flatten[Table[ 
If[doubleReaction[[i]]== 0,reacs[[i]],
{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}],{i,n}] ];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)

(* Parameters ki of all the reactions *)
parametros=Flatten[Table[If[doubleReaction[[i]]== 0,Subscript[Global`k, i],
{Subscript[Global`k, i],Subscript[Global`k, -i]}]  ,
{i,n}] ];
n=Length[reacs];

(* Display for control
Do[ Print[reactions[[i,1]]," ",ta[[i]]," ",reactions[[i,2]]],{i,n}];
*)

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,{If[StringMatchQ[ToString[var[[i]]],"*$"],
(**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
reacs=reacs/.var[[i]]->tmp;
var[[i]]=tmp;
Do[If[var[[i]]==var[[j]],(**If found**){isSteady[[j]]=True;
var=Drop[var,{i}];
m--;
i--;
Goto[fim];}],{j,i-1,1,-1}];(**End Do**)isSteady=Append[isSteady,True];},
isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

(* Mass Action*)
Do[{Alpha[i,j]=Coefficient[reacs[[i,1]],var[[j]]],
Beta[i,j]=Coefficient[reacs[[i,2]],var[[j]]]},{i,n},{j,m}];(**End Do**)

Gamma=Table[Beta[i,j]-Alpha[i,j],{i,n},{j,m}];
Do[Omega[i]=parametros[[i]]*Product[var[[j]]^Alpha[i,j],{j,m}],{i,n}];

output=tmp={};

Do[If[isSteady[[j]],tmp=Append[tmp,var[[j]][Global`t]->var[[j]]];,
output=Append[output,D[var[[j]][Global`t],Global`t]==
Sum[Gamma[[i,j]]*parametros[[i]]*Product[var[[j]][Global`t]^Alpha[i,j],{j,m}],
{i,n}] ]],{j,m}];(** End Do **) 
output=output/.tmp;
Return[output];
]; (* End of EquationsKinetics[reactions] *)


ParameterNamesKinetics[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction, parametros,paraux,isSteady,tmp,j,output}, 

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],doubleReaction=Append[doubleReaction,1],doubleReaction=Append[doubleReaction,0]],{i,n}];
m1=Sum[doubleReaction[[i]],{i,n}];
reacs=Flatten[Table[ 
If[doubleReaction[[i]]== 0,reacs[[i]],{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}]  ,
{i,n}] ];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)

(* Parameters ki of all the reactions *)
(* parametros=Flatten[Table[If[doubleReaction[[i]]== 0,Subscript[Global`k, i],
{Subscript[Global`k, i],Subscript[Global`k, -i]}]  ,
{i,n}] ];*)
parametros=Table[Subscript[Global`k, i] ,{i,n}] ;
paraux={};
Do[If[doubleReaction[[i]]== 1,paraux=Append[paraux,Subscript[Global`k, -i]]],{i,n}];
(*paraux=Table[If[doubleReaction[[i]]== 1,Subscript[Global`k, -i]]  ,
{i,n}] ;*)
parametros=Join[parametros,paraux];
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,{If[StringMatchQ[ToString[var[[i]]],"*$"],
(**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
reacs=reacs/.var[[i]]->tmp;
var[[i]]=tmp;
Do[If[var[[i]]==var[[j]],(**If found**){isSteady[[j]]=True;
var=Drop[var,{i}];
m--;
i--;
Goto[fim];}],{j,i-1,1,-1}];(**End Do**)isSteady=Append[isSteady,True];},
isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

Do[ If[isSteady[[i]],parametros=Append[parametros,var[[i]]]],{i,m}];
output=parametros;

Return[output];
];(* End of ParameterNamesKinetics[reactions] *)


ParameterInputKinetics[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction, parametros,paraux,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],doubleReaction=Append[doubleReaction,1],doubleReaction=Append[doubleReaction,0]],{i,n}];
m1=Sum[doubleReaction[[i]],{i,n}];
reacs=Flatten[Table[ 
If[doubleReaction[[i]]== 0,reacs[[i]],{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}]  ,
{i,n}] ];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)

(* Parameters ki of all the reactions *)
(* parametros=Flatten[Table[If[doubleReaction[[i]]== 0,Subscript[Global`k, i],
{Subscript[Global`k, i],Subscript[Global`k, -i]}]  ,
{i,n}] ];*)
parametros=Table[Subscript[Global`k, i] ,{i,n}] ;
paraux={};
Do[If[doubleReaction[[i]]== 1,paraux=Append[paraux,Subscript[Global`k, -i]]],{i,n}];
(*paraux=Table[If[doubleReaction[[i]]== 1,Subscript[Global`k, -i]]  ,
{i,n}] ;*)
parametros=Join[parametros,paraux];
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,{If[StringMatchQ[ToString[var[[i]]],"*$"],
(**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
reacs=reacs/.var[[i]]->tmp;
var[[i]]=tmp;
Do[If[var[[i]]==var[[j]],(**If found**){isSteady[[j]]=True;
var=Drop[var,{i}];
m--;
i--;
Goto[fim];}],{j,i-1,1,-1}];(**End Do**)isSteady=Append[isSteady,True];},
isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

Do[ If[isSteady[[i]],parametros=Append[parametros,var[[i]]]],{i,m}];
Do[ parametros[[i]]= parametros[[i]]-> Global`ParInputVar[i],{i,Length[parametros]}];
output=parametros;

Return[output];
];(* End of ParameterInputKinetics[reactions] *)


ReactionGraphKinetics[reactions_]:=
Block[{reacs,n,var,i,m,doubleReaction,ta},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
doubleReaction=Append[doubleReaction,1],
doubleReaction=Append[doubleReaction,0]],{i,n}];(*End Do*)
ta=Table[If[doubleReaction[[i]]==0,
DisplayForm@OverscriptBox["\[LongRightArrow]",Subscript[Global`k, i]],
DisplayForm@UnderoverscriptBox["\[RightArrowLeftArrow]",Subscript[Global`k, -i],Subscript[Global`k, i]]],
{i,n}];(**End Table**)
Do[ Print[reactions[[i,1]]," ",ta[[i]]," ",reactions[[i,2]]],{i,n}];
]; (* End of ReactionGraphKinetics[reactions] *)


SubstanceVariablesKinetics[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
doubleReaction=Append[doubleReaction,1],
doubleReaction=Append[doubleReaction,0]],{i,n}];(*End Do*)

m1=Sum[doubleReaction[[i]],{i,n}]; (* Number of constant variables *)
reacs=Flatten[Table[ 
If[doubleReaction[[i]]== 0,reacs[[i]],
{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}],{i,n}] ];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,{If[StringMatchQ[ToString[var[[i]]],"*$"],
(**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
reacs=reacs/.var[[i]]->tmp;
var[[i]]=tmp;
Do[If[var[[i]]==var[[j]],(**If found**){isSteady[[j]]=True;
var=Drop[var,{i}];
m--;
i--;
Goto[fim];}],{j,i-1,1,-1}];(**End Do**)isSteady=Append[isSteady,True];},
isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

output=var;
Do[ If[isSteady[[i]],output=Drop[output,{i}]],{i,m,1,-1}];
n=Length[output];
Do[ output[[i]]=output[[i]]/.output[[i]]->output[[i]][Global`t],{i,n}];
Return[output];
]; (* End of SubstanceVariablesKinetics[reactions] *)


SubstanceNamesKinetics[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
doubleReaction=Append[doubleReaction,1],
doubleReaction=Append[doubleReaction,0]],{i,n}];(*End Do*)

m1=Sum[doubleReaction[[i]],{i,n}]; (* Number of constant variables *)
reacs=Flatten[Table[ 
If[doubleReaction[[i]]== 0,reacs[[i]],
{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}],{i,n}] ];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,{If[StringMatchQ[ToString[var[[i]]],"*$"],
(**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
reacs=reacs/.var[[i]]->tmp;
var[[i]]=tmp;
Do[If[var[[i]]==var[[j]],(**If found**){isSteady[[j]]=True;
var=Drop[var,{i}];
m--;
i--;
Goto[fim];}],{j,i-1,1,-1}];(**End Do**)isSteady=Append[isSteady,True];},
isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

output=var;
Do[ If[isSteady[[i]],output=Drop[output,{i}]],{i,m,1,-1}];
n=Length[output];
Do[ output[[i]]=output[[i]]/.output[[i]]->output[[i]],{i,n}];
Return[output];
]; (* End of SubstanceNamesKinetics[reactions] *)


SubstanceInitialConditionsKinetics[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,isSteady,tmp,j,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
doubleReaction=Append[doubleReaction,1],
doubleReaction=Append[doubleReaction,0]],{i,n}];(*End Do*)

m1=Sum[doubleReaction[[i]],{i,n}]; (* Number of constant variables *)
reacs=Flatten[Table[ 
If[doubleReaction[[i]]== 0,reacs[[i]],
{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}],{i,n}] ];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,{If[StringMatchQ[ToString[var[[i]]],"*$"],
(**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
reacs=reacs/.var[[i]]->tmp;
var[[i]]=tmp;
Do[If[var[[i]]==var[[j]],(**If found**){isSteady[[j]]=True;
var=Drop[var,{i}];
m--;
i--;
Goto[fim];}],{j,i-1,1,-1}];(**End Do**)isSteady=Append[isSteady,True];},
isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

output=var;
Do[ If[isSteady[[i]],output=Drop[output,{i}]],{i,m,1,-1}];
n=Length[output];
Do[ output[[i]]=output[[i]]/.output[[i]]->output[[i]][0]== Global`InitCond[i],{i,n}];
Return[output];
]; (* End of SubstanceInitialConditionsKinetics[reactions] *)


ConservationLawsKinetics[reactions_]:=
Block[{reacs,n,var,i,m,m1,doubleReaction,parametros,isSteady,tmp,j,Alpha,Beta,Gamma,Omega,Sigma,output},

reacs=reactions;

n=Length[reacs];
var=Union@Flatten@Table[{Variables@reacs[[i,1]],Variables@reacs[[i,2]]},{i,n}];
m=Length[var];
(* var contains the variables of all the reactions, including the special symbol case, *$  *)

(* Find the reactions in both directions *)
doubleReaction={};
Do[
If[StringMatchQ[ToString[reactions[[i]]],"*\[LeftRightArrow]*"],
doubleReaction=Append[doubleReaction,1],
doubleReaction=Append[doubleReaction,0]],{i,n}];(*End Do*)
m1=Sum[doubleReaction[[i]],{i,n}]; (* Number of constant variables *)
reacs=Flatten[Table[ 
If[doubleReaction[[i]]== 0,reacs[[i]],
{reacs[[i,1]]-> reacs[[i,2]],reacs[[i,2]]-> reacs[[i,1]]}],{i,n}] ];
(* The string reacs[[*]] contains  the inverse reactions and its
length is different from the length of reactions[[*]] *)

(* Parameters ki of all the reactions *)
parametros=Flatten[Table[If[doubleReaction[[i]]== 0,Subscript[Global`k, i],
{Subscript[Global`k, i],Subscript[Global`k, -i]}]  ,
{i,n}] ];
n=Length[reacs];

(* Now, we determine the variables that must be considered constant.
They are indicated in the input by the symbol Name$ *)
isSteady={};
For[i=1,i<=m,i++,{If[StringMatchQ[ToString[var[[i]]],"*$"],
(**If True**){tmp=ToExpression[StringDrop[ToString[var[[i]]],-1]];
reacs=reacs/.var[[i]]->tmp;
var[[i]]=tmp;
Do[If[var[[i]]==var[[j]],(**If found**){isSteady[[j]]=True;
var=Drop[var,{i}];
m--;
i--;
Goto[fim];}],{j,i-1,1,-1}];(**End Do**)isSteady=Append[isSteady,True];},
isSteady=Append[isSteady,False];];
Label[fim];}]; (**End For**)
(* At this point, the names of the chemical symbols in var do not contain the symbol $. isSteady is a string containing "True" and "False" if, in its position i, the corresponding variables in var is Steady (constant) or not. *)

(* Mass Action*)
Do[{Alpha[i,j]=Coefficient[reacs[[i,1]],var[[j]]],
Beta[i,j]=Coefficient[reacs[[i,2]],var[[j]]]},{i,n},{j,m}];(**End Do**)

Gamma=Table[Beta[i,j]-Alpha[i,j],{i,n},{j,m}];
Do[If[isSteady[[j]],Do[Gamma[[i,j]]=0,{i,n}] ],{j,m}];

Do[Omega[i]=parametros[[i]]*Product[var[[j]]^Alpha[i,j],{j,m}],{i,n}];
Sigma=NullSpace[Gamma];

output=tmp={};

Do[output=Append[output,Sum[Sigma[[j,i]]var[[i]][Global`t],{i,m}]],{j,Length[Sigma]}];

Do[If[isSteady[[j]],tmp=Append[tmp,var[[j]][Global`t]->var[[j]]]],{j,m}];

(* Print control
Do[Print["     ",Sum[Sigma[[j,i]]var[[i]],{i,m}]," = constant"],{j,Length[Sigma]}];
*)
output=output/.tmp;
Return[output];
]; (* End of ConservationLawsKinetics[reactions] *)


End[];
EndPackage[];
